# ruff: noqa: F405

"""Synchronous wrappers for OxaPay async API client."""

import asyncio
import types
from collections.abc import Callable
from functools import wraps
from typing import TypeVar

from .client import OxaPayClient
from .models import *  # noqa: F403

T = TypeVar("T")


def sync_wrapper(func: Callable[..., T]) -> Callable[..., T]:
    """Convert an async function to a sync function.

    Args:
        func: The async function to convert

    Returns:
        A synchronous wrapper that runs the async function in an event loop

    """

    @wraps(func)
    def wrapper(*args: object, **kwargs: object) -> T:
        try:
            loop = asyncio.get_event_loop()
        except RuntimeError:
            # If there's no event loop in the current thread, create one
            loop = asyncio.new_event_loop()
            asyncio.set_event_loop(loop)

        return loop.run_until_complete(func(*args, **kwargs))

    return wrapper


class SyncOxaPayClient:
    """Synchronous wrapper for OxaPayClient.

    Provides the same methods as OxaPayClient but in a synchronous manner.
    """

    def __init__(
        self,
        merchant_api_key: str | None = None,
        payout_api_key: str | None = None,
        general_api_key: str | None = None,
    ) -> None:
        """Initialize the sync OxaPay client.

        Args:
            merchant_api_key: API key for merchant operations
            payout_api_key: API key for payout operations
            general_api_key: API key for general operations like exchange

        """
        self._async_client = OxaPayClient(
            merchant_api_key=merchant_api_key,
            payout_api_key=payout_api_key,
            general_api_key=general_api_key,
        )

    def __enter__(self) -> "SyncOxaPayClient":
        """Enter context manager.

        Returns:
            The SyncOxaPayClient instance

        """
        return self

    def __exit__(
        self,
        exc_type: type[BaseException] | None,
        exc_val: BaseException | None,
        exc_tb: types.TracebackType | None,
    ) -> None:
        """Exit context manager.

        Args:
            exc_type: Exception type if an exception was raised
            exc_val: Exception value if an exception was raised
            exc_tb: Exception traceback if an exception was raised

        """

    # Merchant API Methods

    @sync_wrapper
    async def create_invoice(
        self,
        request: CreateInvoiceRequest,
    ) -> CreateInvoiceResponse:
        """Create a new payment link.

        Args:
            request: Invoice creation parameters

        Returns:
            Invoice creation response with payment link and tracking ID

        """
        return await self._async_client.create_invoice(request)

    @sync_wrapper
    async def create_white_label_payment(
        self,
        request: CreateWhiteLabelRequest,
    ) -> CreateWhiteLabelResponse:
        """Create a white-labeled payment.

        Args:
            request: White-label payment creation parameters

        Returns:
            White-label payment response with payment details

        """
        return await self._async_client.create_white_label_payment(request)

    @sync_wrapper
    async def create_static_wallet(
        self,
        request: CreateStaticWalletRequest,
    ) -> CreateStaticWalletResponse:
        """Create a static wallet address.

        Args:
            request: Static wallet creation parameters

        Returns:
            Static wallet address creation response

        """
        return await self._async_client.create_static_wallet(request)

    @sync_wrapper
    async def revoke_static_wallet(
        self,
        request: RevokeStaticWalletRequest,
    ) -> RevokeStaticWalletResponse:
        """Revoke a static wallet address.

        Args:
            request: Static wallet revocation parameters

        Returns:
            Static wallet revocation response

        """
        return await self._async_client.revoke_static_wallet(request)

    @sync_wrapper
    async def get_static_wallets(
        self,
        request: StaticWalletListRequest,
    ) -> StaticWalletListResponse:
        """Get a list of static wallets.

        Args:
            request: Static wallet list request parameters

        Returns:
            List of static wallets with pagination info

        """
        return await self._async_client.get_static_wallets(request)

    @sync_wrapper
    async def get_payment_info(self, track_id: str | int) -> PaymentInfoResponse:
        """Get information about a specific payment.

        Args:
            track_id: The payment tracking ID

        Returns:
            Payment information

        """
        return await self._async_client.get_payment_info(track_id)

    @sync_wrapper
    async def get_payment_history(
        self,
        request: PaymentHistoryRequest,
    ) -> PaymentHistoryResponse:
        """Get payment history.

        Args:
            request: Payment history request parameters

        Returns:
            Payment history with pagination info

        """
        return await self._async_client.get_payment_history(request)

    @sync_wrapper
    async def get_accepted_coins(self) -> AcceptedCoinsResponse:
        """Get a list of accepted cryptocurrencies.

        Returns:
            List of accepted cryptocurrency symbols

        """
        return await self._async_client.get_accepted_coins()

    # Payout API Methods

    @sync_wrapper
    async def create_payout(self, request: CreatePayoutRequest) -> CreatePayoutResponse:
        """Create a new payout.

        Args:
            request: Payout creation parameters

        Returns:
            Payout creation response with tracking ID

        """
        return await self._async_client.create_payout(request)

    @sync_wrapper
    async def get_payout_info(self, track_id: str | int) -> PayoutInfoResponse:
        """Get information about a specific payout.

        Args:
            track_id: The payout tracking ID

        Returns:
            Payout information

        """
        return await self._async_client.get_payout_info(track_id)

    @sync_wrapper
    async def get_payout_history(
        self,
        request: PayoutHistoryRequest,
    ) -> PayoutHistoryResponse:
        """Get payout history.

        Args:
            request: Payout history request parameters

        Returns:
            Payout history with pagination info

        """
        return await self._async_client.get_payout_history(request)

    @sync_wrapper
    async def get_account_balance(
        self,
        currency: str | None = None,
    ) -> AccountBalanceResponse:
        """Get account balance for all or a specific currency.

        Args:
            currency: Specific currency to get balance for (optional)

        Returns:
            Account balance information

        """
        return await self._async_client.get_account_balance(currency)

    # Exchange API Methods

    @sync_wrapper
    async def get_exchange_rate(
        self,
        request: ExchangeRateRequest,
    ) -> ExchangeRateResponse:
        """Get exchange rate between two currencies.

        Args:
            request: Exchange rate request parameters

        Returns:
            Exchange rate information

        """
        return await self._async_client.get_exchange_rate(request)

    @sync_wrapper
    async def calculate_exchange(
        self,
        request: ExchangeCalculateRequest,
    ) -> ExchangeCalculateResponse:
        """Calculate exchange amount.

        Args:
            request: Exchange calculation request parameters

        Returns:
            Exchange calculation result

        """
        return await self._async_client.calculate_exchange(request)

    @sync_wrapper
    async def get_exchange_pairs(self) -> ExchangePairsResponse:
        """Get available exchange pairs.

        Returns:
            List of available exchange pairs

        """
        return await self._async_client.get_exchange_pairs()

    @sync_wrapper
    async def create_exchange_request(
        self,
        request: ExchangeRequestRequest,
    ) -> ExchangeRequestResponse:
        """Create a new exchange request.

        Args:
            request: Exchange request parameters

        Returns:
            Exchange request creation response

        """
        return await self._async_client.create_exchange_request(request)

    @sync_wrapper
    async def get_exchange_history(
        self,
        request: ExchangeHistoryRequest,
    ) -> ExchangeHistoryResponse:
        """Get exchange history.

        Args:
            request: Exchange history request parameters

        Returns:
            Exchange history with pagination info

        """
        return await self._async_client.get_exchange_history(request)

    # System API Methods

    @sync_wrapper
    async def get_prices(self) -> PriceResponse:
        """Get current prices of all supported cryptocurrencies.

        Returns:
            Current prices information

        """
        return await self._async_client.get_prices()

    @sync_wrapper
    async def get_supported_currencies(self) -> SupportedCurrenciesResponse:
        """Get a list of supported currencies.

        Returns:
            List of supported currencies with network details

        """
        return await self._async_client.get_supported_currencies()

    @sync_wrapper
    async def get_supported_fiat_currencies(self) -> SupportedFiatCurrenciesResponse:
        """Get a list of supported fiat currencies.

        Returns:
            List of supported fiat currencies

        """
        return await self._async_client.get_supported_fiat_currencies()

    @sync_wrapper
    async def get_supported_networks(self) -> SupportedNetworksResponse:
        """Get a list of supported blockchain networks.

        Returns:
            List of supported networks

        """
        return await self._async_client.get_supported_networks()

    @sync_wrapper
    async def check_system_status(self) -> str:
        """Check if the OxaPay API is operational.

        Returns:
            Status message (usually "OK")

        """
        return await self._async_client.check_system_status()

    def verify_webhook_signature(
        self,
        signature: str,
        payload: str,
        *,
        is_payment: bool = True,
    ) -> bool:
        """Verify the HMAC signature of a webhook payload.

        Args:
            signature: The HMAC signature from the webhook header
            payload: The raw webhook payload
            is_payment: Whether this is a payment webhook (True) or payout webhook (False)

        Returns:
            True if the signature is valid, False otherwise

        """  # noqa: E501
        return self._async_client.verify_webhook_signature(
            signature,
            payload,
            is_payment=is_payment,
        )



"""Models for OxaPay Merchant API."""

from enum import Enum

from pydantic import BaseModel, Field

from .base import OrderDirection, PaginatedResponse, ResultResponse, SortField


# Common enums
class PaymentType(str, Enum):
    """Payment type enum."""

    INVOICE = "Invoice"
    WHITE_LABEL = "White-Label"
    STATIC_WALLET = "Static Wallet"


class PaymentStatus(str, Enum):
    """Payment status enum."""

    NEW = "New"
    WAITING = "Waiting"
    CONFIRMING = "Confirming"
    PAID = "Paid"
    EXPIRED = "Expired"
    FAILED = "Failed"


# Invoice creation
class CreateInvoiceRequest(BaseModel):
    """Request model for creating an invoice."""

    amount: int | float = Field(..., description="The amount for the payment")
    currency: str | None = Field(
        None,
        description="Specify if you want the invoice amount calculated with a specific "
        "currency symbol",
    )
    callback_url: str | None = Field(
        None,
        description="The URL where payment information will be sent",
        serialization_alias="callbackUrl",
    )
    under_paid_cover: float | None = Field(
        None,
        ge=0,
        le=60,
        description="Acceptable inaccuracy in payment (0-60.00)",
        serialization_alias="underPaidCover",
    )
    fee_paid_by_payer: int | None = Field(
        None,
        ge=0,
        le=1,
        description="Whether payer covers invoice commission (1) or merchant does (0)",
        serialization_alias="feePaidByPayer",
    )
    life_time: int | None = Field(
        None,
        ge=15,
        le=2880,
        description="Expiration time for payment link in minutes (15-2880)",
        serialization_alias="lifeTime",
    )
    email: str | None = Field(
        None,
        description="Payer's email address for reporting",
    )
    order_id: str | None = Field(
        None,
        description="Unique order ID for reference",
        serialization_alias="orderId",
    )
    description: str | None = Field(
        None,
        description="Order details or additional information",
    )
    return_url: str | None = Field(
        None,
        description="URL for redirecting payer after successful payment",
        serialization_alias="returnUrl",
    )


class CreateInvoiceResponse(ResultResponse):
    """Response model for invoice creation."""

    track_id: str = Field(
        ...,
        description="Unique identifier for the payment session",
        validation_alias="trackId",
    )
    pay_link: str = Field(
        ...,
        description="Payment page link for the payer",
        validation_alias="payLink",
    )


# White-label payment
class CreateWhiteLabelRequest(BaseModel):
    """Request model for creating a white-label payment."""

    amount: int | float = Field(..., description="The amount for the payment")
    pay_currency: str = Field(
        ...,
        description="Specific currency for payment",
        serialization_alias="payCurrency",
    )
    currency: str | None = Field(
        None,
        description="Specify if you want the invoice amount calculated with a specific "
        "currency symbol",
    )
    network: str | None = Field(
        None,
        description="Blockchain network for the payment",
    )
    email: str | None = Field(None, description="Payer's email address")
    order_id: str | None = Field(
        None,
        description="Unique order ID for reference",
        serialization_alias="orderId",
    )
    description: str | None = Field(None, description="Additional information")
    callback_url: str | None = Field(
        None,
        description="URL for payment notifications",
        serialization_alias="callbackUrl",
    )
    under_paid_cover: float | None = Field(
        None,
        ge=0,
        le=60,
        description="Acceptable inaccuracy in payment (0-60.00)",
        serialization_alias="underPaidCover",
    )
    fee_paid_by_payer: int | None = Field(
        None,
        ge=0,
        le=1,
        description="Whether payer covers invoice commission (1) or merchant does (0)",
        serialization_alias="feePaidByPayer",
    )
    life_time: int | None = Field(
        None,
        ge=15,
        le=2880,
        description="Expiration time in minutes (15-2880)",
        serialization_alias="lifeTime",
    )


class CreateWhiteLabelResponse(ResultResponse):
    """Response model for white-label payment creation."""

    track_id: str = Field(
        ...,
        description="Unique identifier for the payment session",
        validation_alias="trackId",
    )
    amount: int | float = Field(..., description="Requested payment amount")
    currency: str = Field(..., description="Requested payment currency")
    pay_amount: int | float = Field(
        ...,
        description="Actual payment amount",
        validation_alias="payAmount",
    )
    pay_currency: str = Field(
        ...,
        description="Actual payment currency",
        validation_alias="payCurrency",
    )
    network: str = Field(..., description="Blockchain network")
    address: str = Field(..., description="Generated cryptocurrency address")
    callback_url: str | None = Field(
        None,
        description="Callback URL",
        validation_alias="callbackUrl",
    )
    description: str | None = Field(None, description="Description")
    email: str | None = Field(None, description="Email")
    fee_paid_by_payer: float = Field(
        ...,
        description="Fee paid by payer setting",
        validation_alias="feePaidByPayer",
    )
    life_time: int = Field(
        ...,
        description="Payment lifetime",
        validation_alias="lifeTime",
    )
    order_id: str | None = Field(
        None,
        description="Order ID",
        validation_alias="orderId",
    )
    under_paid_cover: float = Field(
        ...,
        description="Underpaid cover setting",
        validation_alias="underPaidCover",
    )
    rate: float = Field(..., description="Exchange rate")
    expired_at: str = Field(
        ...,
        description="Expiration time",
        validation_alias="expiredAt",
    )
    created_at: str = Field(
        ...,
        description="Creation time",
        validation_alias="createdAt",
    )
    qr_code: str = Field(
        ...,
        description="QR code image link",
        validation_alias="QRCode",
    )


# Static wallet
class CreateStaticWalletRequest(BaseModel):
    """Request model for creating a static wallet."""

    currency: str = Field(..., description="Currency for the static address")
    network: str | None = Field(None, description="Blockchain network")
    callback_url: str | None = Field(
        None,
        description="URL for payment notifications",
        serialization_alias="callbackUrl",
    )
    email: str | None = Field(None, description="Email address")
    order_id: str | None = Field(
        None,
        description="Order ID for reference",
        serialization_alias="orderId",
    )
    description: str | None = Field(None, description="Additional information")


class CreateStaticWalletResponse(ResultResponse):
    """Response model for static wallet creation."""

    address: str = Field(..., description="Generated static address")


class RevokeStaticWalletRequest(BaseModel):
    """Request model for revoking a static wallet."""

    address: str = Field(..., description="Address of the static wallet to revoke")


class RevokeStaticWalletResponse(ResultResponse):
    """Response model for static wallet revocation."""


class StaticWallet(BaseModel):
    """Model for static wallet information."""

    track_id: str = Field(
        ...,
        description="Unique identifier",
        serialization_alias="trackId",
    )
    address: str = Field(..., description="Static address")
    network: str = Field(..., description="Blockchain network")
    callback_url: str | None = Field(
        None,
        description="Callback URL",
        validation_alias="callbackUrl",
    )
    email: str | None = Field(None, description="Email")
    order_id: str | None = Field(
        None,
        description="Order ID",
        validation_alias="orderId",
    )
    description: str | None = Field(None, description="Description")
    date: str = Field(..., description="Creation timestamp")


class StaticWalletListRequest(BaseModel):
    """Request model for listing static wallets."""

    track_id: str | None = Field(
        None,
        description="Filter by specific invoice ID",
        serialization_alias="trackId",
    )
    page: int | None = Field(1, ge=1, description="Page number")
    size: int | None = Field(10, ge=1, le=200, description="Records per page")
    network: str | None = Field(None, description="Filter by blockchain network")
    address: str | None = Field(None, description="Filter by address")
    email: str | None = Field(None, description="Filter by email")
    order_id: str | None = Field(
        None,
        description="Filter by order ID",
        serialization_alias="orderId",
    )


class StaticWalletListResponse(ResultResponse):
    """Response model for listing static wallets."""

    data: list[StaticWallet] = Field(..., description="List of static wallets")
    meta: dict[str, int] = Field(..., description="Pagination metadata")


# Payment information
class PaymentInfoRequest(BaseModel):
    """Request model for getting payment information."""

    track_id: str = Field(
        ...,
        description="Unique identifier of the payment session",
        serialization_alias="trackId",
    )


class PaymentInfoResponse(ResultResponse):
    """Response model for payment information."""

    track_id: str = Field(
        ...,
        description="Unique identifier of the payment session",
        validation_alias="trackId",
    )
    status: PaymentStatus = Field(..., description="Current status of the payment")
    type: str = Field(..., description="Invoice type")
    amount: str = Field(..., description="Total payment amount")
    currency: str = Field(..., description="Payment currency")
    pay_amount: str = Field(
        ...,
        description="Actual amount paid",
        validation_alias="payAmount",
    )
    pay_currency: str = Field(
        ...,
        description="Currency in which payment was made",
        validation_alias="payCurrency",
    )
    network: str = Field(..., description="Blockchain network")
    address: str = Field(..., description="Payment address")
    tx_id: str | None = Field(
        None,
        description="Transaction ID",
        validation_alias="txID",
    )
    email: str | None = Field(None, description="Customer email")
    order_id: str | None = Field(
        None,
        description="Order ID",
        validation_alias="orderId",
    )
    description: str | None = Field(None, description="Description")
    fee_paid_by_payer: int = Field(
        ...,
        description="Fee paid by payer setting",
        validation_alias="feePaidByPayer",
    )
    under_paid_cover: int = Field(
        ...,
        description="Underpaid cover setting",
        validation_alias="underPaidCover",
    )
    date: str = Field(..., description="Creation time")
    pay_date: str | None = Field(
        None,
        description="Payment time",
        validation_alias="payDate",
    )


# Payment history
class PaymentHistoryRequest(BaseModel):
    """Request model for getting payment history."""

    order_by: OrderDirection | None = Field(
        OrderDirection.DESC,
        description="Sort order",
        serialization_alias="orderBy",
    )
    sort_by: SortField | None = Field(
        SortField.CREATE_DATE,
        description="Sort field",
        serialization_alias="sortBy",
    )
    track_id: str | None = Field(
        None,
        description="Filter by specific invoice ID",
        serialization_alias="trackId",
    )
    page: int | None = Field(1, ge=1, description="Page number")
    size: int | None = Field(10, ge=1, le=200, description="Records per page")
    order_id: str | None = Field(
        None,
        description="Filter by order ID",
        serialization_alias="orderId",
    )
    status: PaymentStatus | None = Field(None, description="Filter by status")
    fee_paid_by_payer: int | None = Field(
        None,
        ge=0,
        le=1,
        description="Filter by fee paid setting",
        serialization_alias="feePaidByPayer",
    )
    type: PaymentType | None = Field(None, description="Filter by payment type")
    network: str | None = Field(None, description="Filter by blockchain network")
    pay_currency: str | None = Field(
        None,
        description="Filter by payment currency",
        serialization_alias="payCurrency",
    )
    currency: str | None = Field(None, description="Filter by currency")
    to_amount: float | None = Field(
        None,
        description="Filter by maximum amount",
        serialization_alias="toAmount",
    )
    from_amount: float | None = Field(
        None,
        description="Filter by minimum amount",
        serialization_alias="fromAmount",
    )
    to_date: int | str | None = Field(
        None,
        description="End date for filtering (unix timestamp)",
        serialization_alias="toDate",
    )
    from_date: int | str | None = Field(
        None,
        description="Start date for filtering (unix timestamp)",
        serialization_alias="fromDate",
    )
    address: str | None = Field(None, description="Filter by address")
    tx_id: str | None = Field(
        None,
        description="Filter by transaction hash",
        serialization_alias="txID",
    )


class Payment(BaseModel):
    """Model for payment information in history."""

    track_id: str = Field(
        ...,
        description="Unique identifier",
        validation_alias="trackId",
    )
    status: PaymentStatus = Field(..., description="Payment status")
    type: str = Field(..., description="Payment type")
    amount: str = Field(..., description="Payment amount")
    currency: str = Field(..., description="Currency")
    pay_amount: str | None = Field(
        None,
        description="Actual amount paid",
        validation_alias="payAmount",
    )
    pay_currency: str | None = Field(
        None,
        description="Payment currency",
        validation_alias="payCurrency",
    )
    network: str | None = Field(None, description="Blockchain network")
    address: str | None = Field(None, description="Payment address")
    tx_id: str | None = Field(
        None,
        description="Transaction ID",
        validation_alias="txID",
    )
    email: str | None = Field(None, description="Email")
    order_id: str | None = Field(
        None,
        description="Order ID",
        validation_alias="orderId",
    )
    description: str | None = Field(None, description="Description")
    date: str = Field(..., description="Creation timestamp")
    pay_date: str | None = Field(
        None,
        description="Payment timestamp",
        validation_alias="payDate",
    )


class PaymentHistoryResponse(PaginatedResponse[Payment]):
    """Response model for payment history."""


# Accepted coins
class AcceptedCoinsRequest(BaseModel):
    """Request model for getting accepted coins."""


class AcceptedCoinsResponse(ResultResponse):
    """Response model for accepted coins."""

    allowed: list[str] = Field(..., description="List of accepted cryptocurrencies")



"""Models for OxaPay Payout API."""

from datetime import datetime
from enum import Enum

from pydantic import BaseModel, Field

from .base import OrderDirection, PaginatedResponse, ResultResponse, SortField


class PayoutStatus(str, Enum):
    """Payout status enum."""

    PROCESSING = "Processing"
    CONFIRMING = "Confirming"
    COMPLETE = "Complete"
    REJECTED = "Rejected"


class PayoutType(str, Enum):
    """Payout type enum."""

    INTERNAL = "internal"
    EXTERNAL = "external"


class CreatePayoutRequest(BaseModel):
    """Request model for creating a payout."""

    address: str = Field(..., description="Cryptocurrency address of the recipient")
    amount: float = Field(..., description="Amount of cryptocurrency to send")
    currency: str = Field(..., description="Cryptocurrency symbol")
    network: str | None = Field(None, description="Blockchain network")
    memo: str | None = Field(None, description="Memo for networks that support it")
    callback_url: str | None = Field(
        None,
        description="URL for payout notifications",
        serialization_alias="callbackUrl",
    )
    description: str | None = Field(None, description="Additional information")


class CreatePayoutResponse(ResultResponse):
    """Response model for payout creation."""

    track_id: str = Field(
        ...,
        description="Unique identifier for the payout",
        validation_alias="trackId",
    )
    status: PayoutStatus = Field(..., description="Current status of the payout")


class PayoutInfoRequest(BaseModel):
    """Request model for getting payout information."""

    track_id: str = Field(
        ...,
        description="Unique identifier of the payout",
        serialization_alias="trackId",
    )


class PayoutInfoResponse(ResultResponse):
    """Response model for payout information."""

    track_id: str = Field(
        ...,
        description="Unique identifier",
        validation_alias="trackId",
    )
    address: str = Field(..., description="Recipient's address")
    currency: str = Field(..., description="Cryptocurrency symbol")
    network: str = Field(..., description="Blockchain network")
    amount: str = Field(..., description="Payout amount")
    fee: str = Field(..., description="Transaction fee")
    status: PayoutStatus = Field(..., description="Current status")
    type: PayoutType = Field(..., description="Payout type")
    tx_id: str | None = Field(
        None,
        description="Transaction ID",
        validation_alias="txID",
    )
    date: str = Field(..., description="Creation timestamp")
    description: str | None = Field(None, description="Description")


class PayoutHistoryRequest(BaseModel):
    """Request model for getting payout history."""

    order_by: OrderDirection | None = Field(
        OrderDirection.DESC,
        description="Sort order",
        serialization_alias="orderBy",
    )
    sort_by: SortField | None = Field(
        SortField.CREATE_DATE,
        description="Sort field",
        serialization_alias="sortBy",
    )
    page: int | None = Field(1, ge=1, description="Page number")
    size: int | None = Field(10, ge=1, le=200, description="Records per page")
    status: PayoutStatus | None = Field(None, description="Filter by status")
    type: PayoutType | None = Field(None, description="Filter by type")
    network: str | None = Field(None, description="Filter by network")
    currency: str | None = Field(None, description="Filter by currency")
    to_amount: str | None = Field(
        None,
        description="Maximum amount",
        serialization_alias="toAmount",
    )
    from_amount: str | None = Field(
        None,
        description="Minimum amount",
        serialization_alias="fromAmount",
    )
    to_date: datetime | None = Field(
        None,
        description="End date",
        serialization_alias="toDate",
    )
    from_date: datetime | None = Field(
        None,
        description="Start date",
        serialization_alias="fromDate",
    )


class Payout(BaseModel):
    """Model for payout information in history."""

    id: str = Field(..., description="Unique identifier")
    address: str = Field(..., description="Recipient's address")
    currency: str = Field(..., description="Cryptocurrency symbol")
    network: str = Field(..., description="Blockchain network")
    amount: str = Field(..., description="Payout amount")
    fee: str = Field(..., description="Transaction fee")
    status: PayoutStatus = Field(..., description="Current status")
    type: PayoutType = Field(..., description="Payout type")
    tx_id: str | None = Field(
        None,
        description="Transaction ID",
        validation_alias="txID",
    )
    date: datetime = Field(..., description="Creation timestamp")
    description: str | None = Field(None, description="Description")


class PayoutHistoryResponse(PaginatedResponse[Payout]):
    """Response model for payout history."""



"""Models for OxaPay Exchange API."""

from enum import Enum

from pydantic import BaseModel, Field

from .base import OrderDirection, PaginatedResponse, ResultResponse, SortField


class ExchangeType(str, Enum):
    """Exchange type enum."""

    AUTO_CONVERT = "autoConvert"
    MANUAL_SWAP = "manualSwap"
    SWAP_BY_API = "swapByApi"


class ExchangePair(BaseModel):
    """Model for exchange pair information."""

    from_currency: str = Field(
        ...,
        validation_alias="fromCurrency",
        description="Source currency symbol",
    )
    to_currency: str = Field(
        ...,
        validation_alias="toCurrency",
        description="Target currency symbol",
    )
    min_amount: str = Field(
        ...,
        validation_alias="minAmount",
        description="Minimum conversion amount",
    )


class ExchangeRateRequest(BaseModel):
    """Request model for exchange rate."""

    from_currency: str = Field(
        ...,
        validation_alias="fromCurrency",
        description="Source currency symbol",
    )
    to_currency: str = Field(
        ...,
        validation_alias="toCurrency",
        description="Target currency symbol",
    )


class ExchangeRateResponse(ResultResponse):
    """Response model for exchange rate."""

    rate: str = Field(..., description="Exchange rate")


class ExchangeCalculateRequest(BaseModel):
    """Request model for exchange calculation."""

    from_currency: str = Field(
        ...,
        serialization_alias="fromCurrency",
        description="Source currency symbol",
    )
    to_currency: str = Field(
        ...,
        serialization_alias="toCurrency",
        description="Target currency symbol",
    )
    amount: float = Field(..., description="Amount to convert")


class ExchangeCalculateResponse(ResultResponse):
    """Response model for exchange calculation."""

    rate: str = Field(..., description="Exchange rate")
    amount: str = Field(..., description="Input amount")
    to_amount: str = Field(
        ...,
        validation_alias="toAmount",
        description="Resulting amount",
    )


class ExchangePairsResponse(ResultResponse):
    """Response model for exchange pairs."""

    pairs: list[ExchangePair] = Field(..., description="Available exchange pairs")


class ExchangeRequestRequest(BaseModel):
    """Request model for exchange request."""

    from_currency: str = Field(
        ...,
        serialization_alias="fromCurrency",
        description="Source currency symbol",
    )
    to_currency: str = Field(
        ...,
        serialization_alias="toCurrency",
        description="Target currency symbol",
    )
    amount: float = Field(..., description="Amount to convert")


class ExchangeRequestResponse(ResultResponse):
    """Response model for exchange request."""

    track_id: str = Field(
        ...,
        validation_alias="trackId",
        description="Exchange transaction ID",
    )
    status: str = Field(..., description="Exchange status")
    amount: str = Field(..., description="Input amount")
    from_currency: str = Field(
        ...,
        validation_alias="fromCurrency",
        description="Source currency",
    )
    to_currency: str = Field(
        ...,
        validation_alias="toCurrency",
        description="Target currency",
    )
    to_amount: str = Field(
        ...,
        validation_alias="toAmount",
        description="Resulting amount",
    )
    rate: str = Field(..., description="Exchange rate")
    date: str = Field(..., description="Exchange timestamp")


class ExchangeHistoryRequest(BaseModel):
    """Request model for exchange history."""

    order_by: OrderDirection | None = Field(
        OrderDirection.DESC,
        serialization_alias="orderBy",
        description="Sort order",
    )
    sort_by: SortField | None = Field(
        SortField.CREATE_DATE,
        serialization_alias="sortBy",
        description="Sort field",
    )
    page: int | None = Field(1, ge=1, description="Page number")
    size: int | None = Field(10, ge=1, le=200, description="Records per page")
    type: ExchangeType | None = Field(None, description="Filter by exchange type")
    to_currency: str | None = Field(
        None,
        serialization_alias="toCurrency",
        description="Filter by target currency",
    )
    from_currency: str | None = Field(
        None,
        serialization_alias="fromCurrency",
        description="Filter by source currency",
    )
    to_date: int | str | None = Field(
        None,
        serialization_alias="toDate",
        description="End date (unix timestamp)",
    )
    from_date: int | str | None = Field(
        None,
        serialization_alias="fromDate",
        description="Start date (unix timestamp)",
    )
    track_id: int | None = Field(
        None,
        serialization_alias="trackId",
        description="Filter by transaction ID",
    )


class Exchange(BaseModel):
    """Model for exchange information in history."""

    track_id: str = Field(
        ...,
        validation_alias="trackId",
        description="Exchange transaction ID",
    )
    from_currency: str = Field(
        ...,
        validation_alias="fromCurrency",
        description="Source currency",
    )
    to_currency: str = Field(
        ...,
        validation_alias="toCurrency",
        description="Target currency",
    )
    from_amount: str = Field(
        ...,
        validation_alias="fromAmount",
        description="Input amount",
    )
    to_amount: str = Field(
        ...,
        validation_alias="toAmount",
        description="Resulting amount",
    )
    rate: str = Field(..., description="Exchange rate")
    type: ExchangeType = Field(..., description="Exchange type")
    date: str = Field(..., description="Exchange timestamp")


class ExchangeHistoryResponse(PaginatedResponse[Exchange]):
    """Response model for exchange history."""



"""Models for OxaPay System API."""

from typing import Any

from pydantic import BaseModel, Field

from .base import ResultResponse


class NetworkInfo(BaseModel):
    """Model for network information."""

    network: str = Field(..., description="Network identifier")
    name: str = Field(..., description="Network name")
    min_confirm: int = Field(
        ...,
        description="Minimum confirmations required",
        validation_alias="minConfirm",
    )
    withdraw_fee: float = Field(
        ...,
        description="Withdrawal fee",
        validation_alias="withdrawFee",
    )
    withdraw_min: float = Field(
        ...,
        description="Minimum withdrawal amount",
        validation_alias="withdrawMin",
    )
    deposit_min: float = Field(
        ...,
        description="Minimum deposit amount",
        validation_alias="depositMin",
    )
    static_fixed_fee: float = Field(
        ...,
        description="Static fixed fee",
        validation_alias="staticFixedFee",
    )


class CurrencyInfo(BaseModel):
    """Model for currency information."""

    symbol: str = Field(..., description="Currency symbol")
    name: str = Field(..., description="Currency name")
    status: bool = Field(..., description="Currency status")
    network_list: dict[str, NetworkInfo] = Field(
        ...,
        description="Available networks",
        validation_alias="networkList",
    )


class SupportedCurrenciesResponse(ResultResponse):
    """Response model for supported currencies."""

    data: dict[str, CurrencyInfo] = Field(
        ...,
        description="Supported currencies information",
    )


class FiatCurrencyInfo(BaseModel):
    """Model for fiat currency information."""

    symbol: str = Field(..., description="Currency symbol")
    name: str = Field(..., description="Currency name")
    price: str = Field(..., description="Current price")
    display_precision: str = Field(
        ...,
        description="Display precision",
        validation_alias="displayPrecision",
    )


class SupportedFiatCurrenciesResponse(ResultResponse):
    """Response model for supported fiat currencies."""

    data: dict[str, FiatCurrencyInfo] = Field(
        ...,
        description="Supported fiat currencies information",
    )


class SupportedNetworksResponse(ResultResponse):
    """Response model for supported networks."""

    data: list[str] = Field(..., description="List of supported network names")


class PriceResponse(ResultResponse):
    """Response model for cryptocurrency prices."""

    data: dict[str, Any] = Field(
        ...,
        description="Current prices of supported cryptocurrencies",
    )
